/**
  * ircommander.c
  * 
  * Dit programma leest de IR-signalen van panasonic-afstandsbedieningen
	* en voert de bijbehorende commando's uit zoals opgegeven in
	* config/ircommander.conf
  * 
	* Het programma beschouwt het PRU-geheugen als een grote
	* uint64-array met de lengte op index 0.
  */

#include <stdio.h>
#include <sys/mman.h>
#include <unistd.h>
#include <string.h>
#include <prussdrv.h>
#include <pruss_intc_mapping.h>
#include <getopt.h>
#include <glib.h>

#ifdef __DEBUG
#define DEBUG_PRINTF(FORMAT, ...) fprintf(stderr, FORMAT, ## __VA_ARGS__)
#else
#define DEBUG_PRINTF(FORMAT, ...)
#endif

tpruss_intc_initdata pruss_intc_initdata = PRUSS_INTC_INITDATA;
GKeyFile *keyfile;
GKeyFileFlags flags = G_KEY_FILE_NONE;
GError *error;
int programming_mode = 0;

struct option options[] = {
	{"help"   , no_argument, 0, 'h'},
	{"program", no_argument, 0, 'p'},
	{"config" , required_argument, 0, 'c'}
};

void poll(void *addr) {
 	unsigned int index = 0, repetition = 0;
	unsigned int *length = addr;
	unsigned long long *array = (unsigned long long*)(length+1);
	
	unsigned long long keycode, previous_keycode;
	char keycode_string[20];
	char *command;
	size_t command_length = 200;
	char *newline_address;
	
	while (1) {
		// Wacht op een interrupt van de PRU
    prussdrv_pru_wait_event (PRU_EVTOUT_0);
    prussdrv_pru_clear_event (PRU_EVTOUT_0, PRU0_ARM_INTERRUPT);
		
		// Lees alle nieuwe commando's
		for (; index < *length; ++index) {
			keycode = array[index & ((1<<8)-1)];
			sprintf(keycode_string, "%llX", keycode);

			// Vergeef de gebruiker die de knop iets te lang indrukt
			if (keycode == previous_keycode) {
				if (++repetition <= 2)
					continue;
			}
			else {
				previous_keycode = keycode;
				repetition = 0;
			}
			
			DEBUG_PRINTF(stderr, "Nieuwe keycode %s.\n", keycode_string);

			if (!g_key_file_has_key(keyfile, "Commands", keycode_string, NULL) && programming_mode) {
				fprintf(stderr, "Voer het commando voor %s in:\n", keycode_string);
				command = g_malloc(command_length);
				
				fprintf(stderr, "keycode_string r %d is %s \n", __LINE__, keycode_string);
				if (getline(&command, &command_length, stdin) != -1) {
					fprintf(stderr, "keycode_string r %d is %s \n", __LINE__, keycode_string);
					// Haal de newline uit het commando
					if (newline_address = strchr(command, '\n'))
						*newline_address = '\0';

					fprintf(stderr, "keycode_string r %d is %s \n", __LINE__, keycode_string);
					fprintf(stderr, "g_key_file_set_string(keyfile, \"Commands\", \"%s\", \"%s\");\n", keycode_string, command);
					g_key_file_set_string(keyfile, "Commands", keycode_string, command);
					fprintf(stderr, "g_key_file_save_to_file(keyfile, \"../config/ircommander.conf\", NULL);\n");
					g_key_file_save_to_file(keyfile, "../config/ircommander.conf", NULL);
				}
				g_free(command);
			}
			
			if (g_key_file_has_key(keyfile, "Commands", keycode_string, NULL)) {
				command = g_key_file_get_string(keyfile, "Commands", keycode_string, NULL);
				printf("%s\n", command);
				fflush(stdout); // het programma wil niet flushen ondanks de newline
				DEBUG_PRINTF(stderr, "Commando \"%s\" verstuurd.\n", command);
				g_free(command);
			}
		}
	}
}

int main(int argc, char *argv[])
{
	switch (getopt_long(argc, argv, "hpc:", options, NULL)) {
		case 'h':
			printf("ircommander [-h | --help] [-p | --program] [-cfile | --config file]\n");
			printf("-h      display this help\n");
			printf("-p      bind commands to new keys\n");
			printf("-c file select a different config file [not implemented]\n");
			return 0;
		break;
		
		case 'p':
			programming_mode = 1;
			fprintf(stderr, "Programma geopend voor nieuwe toetsencombinaties\n");
		break;
	}
	
	void *prumem_address;
/*
	// Koppel het programma los van de sessie d.m.v. een fork
	pid_t pid = fork();

	if (pid < 0)
		exit(EXIT_FAILURE);

	if (pid > 0) {
		printf("Module gestart met pid %d", pid);
		exit(EXIT_SUCCESS);
	}

	if (setsid() < 0)
		exit(EXIT_FAILURE);
*/

	keyfile = g_key_file_new();
	if (!g_key_file_load_from_file(keyfile, "../config/ircommander.conf", flags, &error)) {
		fprintf(stderr, error->message);
	}

	prussdrv_init();

	// Open de PRU zonder speciale interrupts en laad de mmap
	if (prussdrv_open(PRU_EVTOUT_0))
	{
		fprintf(stderr, "prussdrv_open open failed\n");
		return 1;
	}

	// Start de interrupt controller
	prussdrv_pruintc_init(&pruss_intc_initdata);
	
	// Maak een koppeling met het geheugen van de PRU
	prussdrv_map_prumem(PRUSS0_PRU0_DATARAM, &prumem_address);
	
	// Zet de lengte van de array expliciet op 0
	((unsigned int*)prumem_address)[0] = 0;
	
	// Zet het PRU-programma aan
	prussdrv_exec_program(PRU0, "./ircommander.bin");
	
	// Start het pollen van het PRU-geheugen
	poll(prumem_address);

	// Zet de PRU uit en stop de driver
	prussdrv_pru_disable(PRU0); 
	prussdrv_exit();
	
	return 0;
}
