/**
  * rawtiming.c
  * 
  * Dit programma leest de stand van de infraroodsensor uit en
  * zet de tijd tussen wisselingen op de standaarduitvoer.
	* Bit 30 geeft de nieuwe stand van de sensor aan (laag of hoog).
  * 
	* Het programma beschouwt het PRU-geheugen als een grote
	* uint32-array met de lengte op index 0.
  */

#include <stdio.h>
#include <sys/mman.h>
#include <unistd.h>
#include <prussdrv.h>
#include <pruss_intc_mapping.h>

tpruss_intc_initdata pruss_intc_initdata = PRUSS_INTC_INITDATA;

void poll(unsigned int *addr) {
 	int offset = 1;
	int debugprint = 0;
	
	while (1) {
		//Controleer of er nieuwe data klaar staat
		//en zet het op het scherm
		for (; offset <= addr[0]; ++offset)
			printf("%d: %u, ", offset, addr[offset]);
		
		//Print elke halve seconde wat debuginformatie
		if (++debugprint > 500) {
			debugprint = 0;
			printf("length: %u, ", addr[0]);
		}
		
		usleep(1000);
	}
}

int main (void)
{
	unsigned int *prumem_address;
	
	prussdrv_init();

	//Open de PRU zonder uitgaande events en laad de mmaps
	if (prussdrv_open(PRU_EVTOUT_0))
	{
		printf("prussdrv_open open failed\n");
		return 1;
	}

	//Start de interrupt controller
	prussdrv_pruintc_init(&pruss_intc_initdata);
	
	//Maak een koppeling met het geheugen van de PRU
	prussdrv_map_prumem(PRUSS0_PRU0_DATARAM, (void**)&prumem_address);
	
	//Zet de lengte van de array expliciet op 0
	prumem_address[0] = 0;
	
	//Zet het PRU-programma aan
	prussdrv_exec_program(PRU0, "./rawtiming.bin");
	
	//Start het pollen van het PRU-geheugen
	poll(prumem_address);

	//Zet de PRU uit en stop de driver
	prussdrv_pru_disable(PRU0); 
	prussdrv_exit();
	
	return 0;
}
