/**
  * read_signal.c
  * 
  * Dit programma leest de IR-signalen van panasonic-afstandsbedieningen
	* en zet de code op de standaarduitvoer.
  * 
	* Het programma beschouwt het PRU-geheugen als een grote
	* uint64-array met de lengte op index 0.
  */

#include <stdio.h>
#include <sys/mman.h>
#include <unistd.h>
#include <prussdrv.h>
#include <pruss_intc_mapping.h>

tpruss_intc_initdata pruss_intc_initdata = PRUSS_INTC_INITDATA;

void poll(void *addr) {
 	unsigned int index = 0;
	unsigned int *length = addr;
	unsigned long long *array = (unsigned long long*)(length+1);
	
	while (1) {
		//Controleer of er nieuwe data klaar staat
		//en zet het op het scherm
		for (; index < *length; ++index)
			printf("%u: %llX\n", index, array[index & ((1<<8)-1)]);
		
		usleep(1000);
	}
}

int main (void)
{
	void *prumem_address;
	
	prussdrv_init();

	//Open de PRU zonder uitgaande events en laad de mmaps
	if (prussdrv_open(PRU_EVTOUT_0))
	{
		printf("prussdrv_open open failed\n");
		return 1;
	}

	//Start de interrupt controller
	prussdrv_pruintc_init(&pruss_intc_initdata);
	
	//Maak een koppeling met het geheugen van de PRU
	prussdrv_map_prumem(PRUSS0_PRU0_DATARAM, &prumem_address);
	
	//Zet de lengte van de array expliciet op 0
	((unsigned int*)prumem_address)[0] = 0;
	
	//Zet het PRU-programma aan
	prussdrv_exec_program(PRU0, "./read_signal.bin");
	
	//Start het pollen van het PRU-geheugen
	poll(prumem_address);

	//Zet de PRU uit en stop de driver
	prussdrv_pru_disable(PRU0); 
	prussdrv_exit();
	
	return 0;
}
